;**
;** kb_asm.asm:
;**     Interrupt handlers for real- and protected-mode KB INT

.286

_TEXT16	SEGMENT	BYTE PUBLIC 'CODE'
	ASSUME	cs:_TEXT16

	;**
	;** The real-mode interrupt handler is in a 16-bit code segment
	;** so that the assembler will generate the right code.
	;**
	;** We will copy this code to a 16-bit segment in low memory 
	;** and install it as a real mode interrupt handler before it
	;** gets executed.  (There's no distinction between code and
	;** data in real mode.)
	;**
	;** Since the handler doesn't get executed before it gets copied,
	;** putting it in this segment makes the C code simpler.
	;**
	;** The buffer into which we will store the data being sent
	;** to us is also in this segment, for simplicity.  (It's
	;** possible to read and write code segments in real mode.)
	;** We will lay out this segment using fixed offsets, so that
	;** we can omit the data in the executable file.  The symbolic
	;** offsets are only used to reference the segment BEFORE
	;** it has been copied low.
	;**
	;** Both the real mode and protected mode handlers will store
	;** their data in this same buffer (that is, in the copy in
	;** low memory).
	;**
	;** OFFSET    LENGTH     CONTENTS
	;**
	;** 0         <128       Real mode interrupt handler
        ;** 128       1          0e0h da ?
        ;** 129       1          0e1h da ?
        ;** 130       256        Key Buffer
        ;**
	;**


        PUBLIC  rmhandler_
rmhandler_:
        push ds
        push ax
        push bx
        push dx


        in   al, 60h                   ; Read ports so interrupts can go on
        .if al == 0e0h
           mov byte ptr cs:[128],128                     ; e0 war da!
           mov byte ptr cs:[129],0             ; e1 war nicht da!
        .elseif al == 0e1h
              mov byte ptr cs:[129],128                    ; e1 war da!
        .else
            mov bl, al
            and bl, 7fh   ; Index in Key Tabelle
            mov ah, al    ; Tastendruck
            xor bh,bh
            or  bl, byte ptr cs:[128]

            and al, 128
            xor al, 128
            mov byte ptr cs:[130+bx], al

            .if byte ptr cs:[129] != 0 && ah == 1dh  ; if (war_e1_da != 0
                                            ; && wert == 0x1d)
                                            ; keys[0] ^= 0x80;
               xor byte ptr cs:[130], 128
            .endif

            .if ah == 39h        ;  if (wert == 0x39) keys[0] = 0;
               mov byte ptr cs:[130], 0
            .endif

            mov byte ptr cs:[129], 0     ; war_e1_da = 0;
            mov byte ptr cs:[128], 0     ; war_e0_da = 0;

        .endif

        mov  dx,020h  ;     outp(0x20,0x20);
        mov  al,dl
        out  dx,al                           ; Send EOI
        pop  dx
        pop  bx
        pop  ax
        pop  ds
	iret
	ASSUME	cs:NOTHING

_TEXT16	ENDS

.386p

_DATA	SEGMENT DWORD PUBLIC 'DATA'
        EXTRN   _lowmem_seg:WORD
_DATA	ENDS

_TEXT	SEGMENT BYTE PUBLIC 'CODE'
        ASSUME  cs:_TEXT, ds:_DATA
        ASSUME  ds:NOTHING

	;**
	;** The protected-mode interrupt handler is in a 32-bit code
	;** segment.  Even so, we have to be sure to force an IRETD
	;** at the end of the handler, because MASM doesn't generate
	;** one.  This handler will be called on a 32-bit stack by
	;** DOS/4GW.
	;**

	PUBLIC	pmhandler_
pmhandler_:
        push ds
        push ebx
        push eax
        push edx

        mov bx,_DATA
        mov ds,bx

        ASSUME  ds:_DATA

        mov bx, [_lowmem_seg]
        mov ds, bx                          ; Get DS addressability to buffer

        ASSUME  ds:NOTHING

        in   al, 60h                   ; Read ports so interrupts can go on
        .if al == 0e0h
           mov byte ptr ds:[128],128                     ; e0 war da!
           mov byte ptr ds:[129],0             ; e1 war nicht da!
        .elseif al == 0e1h
              mov byte ptr ds:[129],128                    ; e1 war da!
        .else
            mov bl, al
            and bl, 7fh   ; Index in Key Tabelle
            mov ah, al    ; Tastendruck
            xor bh,bh
            or  bl, byte ptr ds:[128]

            and al, 128
            xor al, 128
            mov byte ptr ds:[130+bx], al

            .if byte ptr ds:[129] != 0 && ah == 1dh  ; if (war_e1_da != 0
                                            ; && wert == 0x1d)
                                            ; keys[0] ^= 0x80;
               xor byte ptr ds:[130], 128
            .endif

            .if ah == 39h        ;  if (wert == 0x39) keys[0] = 0;
               mov byte ptr ds:[130], 0
            .endif

            mov byte ptr ds:[129], 0     ; war_e1_da = 0;
            mov byte ptr ds:[128], 0     ; war_e0_da = 0;

        .endif

        mov dx,020h
        mov al,dl
        out dx,al                           ; Send EOI

        pop edx
        pop eax
        pop ebx
        pop ds
	iretd

_TEXT	ENDS

END
